#include <OneWire.h>
#include <DallasTemperature.h>

// Code für eigenen Aufbau

#include <PubSubClient.h>
#include <ESP8266WiFi.h>
#define ONE_WIRE_BUS 2  // DS18B20 pin

//**** Diese Daten müssen angepasst werden ***************

#define WIFI_AP "NW-Kennung"
#define WIFI_PASSWORD "NW-Passwort"

#define TOKEN "ESP8266_DS18B20_TOKEN"

char clientID[] = "DS18B20_LAMPE";
char mqttBroker[] = "192.168.xxx.yyy"; // Adresse des MQTT-Brokers
int subNetIP = xxx; //IP des Subnets
int myIP = zzz; //nur der letzte Block der IP ändert sich
String myHostname = "DS18B20_LAMPE";
String mySensor = "DS18B20";
char mqttSubjectTemperatur[] = "wetter/lampe/temperatur";
char mqttSubjectIP[] = "wetter/lampe/ip";

//**********************************************************

String temperature = "Ablesefehler";

WiFiClient wifiClient;

PubSubClient client(wifiClient);
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature DS18B20(&oneWire);
int status = WL_IDLE_STATUS;

void setup()
{
  //Serial.begin(115200);
  InitWiFi();
  client.setServer( mqttBroker, 1883 );
  if ( !client.connected() ) {
    reconnect();
  }
  getData();
  sendMQTTMessage(mqttSubjectTemperatur, temperature);
  delay(100);
  //Die IP wird mitgeschickt, damit man sie nicht so leicht vergisst ;-)
  // Kann man auch weglassen!
  sendMQTTMessage(mqttSubjectIP, "192.168."+String(subNetIP)+"."+String(myIP));
  client.loop();
  ESP.deepSleep(600000000); //sleep 10 min = 600.000.000 us
}

void loop()
{}

String getFormattedFloat(float x, uint8_t precision) {
  char buffer[10];
  dtostrf(x, 7, precision, buffer);
  return (buffer);
}

void getData() {
  float temp;
  do {
    DS18B20.requestTemperatures();
    //Serial.println("T requested");
    temp = DS18B20.getTempCByIndex(0);
  } while (temp == 85.0 || temp == (-127.0));
  //Serial.println("got DATA");
  temperature = getFormattedFloat(temp, 1);
  //Serial.print("formatted Temperature: ");
  //Serial.println(temperature);
}

void sendMQTTMessage(char mqttSubject[], String message) {
  char payload[100];
  //Serial.print("Sende an MQTT-Broker: ");
  //Serial.print(mqttSubject);
  //Serial.print(" - ");
  //Serial.println(message);
  message.toCharArray( payload, 100 );
  client.publish( mqttSubject, payload , true);
  //Serial.println( payload );
}

void connectToWiFi() {
  WiFi.begin(WIFI_AP, WIFI_PASSWORD);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    //Serial.print(".");
  }
  //Serial.println("Verbunden mit AccessPoint");
}

void InitWiFi()
{
  //Serial.println("Verbinde mit AccessPoint ...");
  // Verbindungsversuch mit WiFi
  IPAddress ip(192, 168, subNetIP, myIP);
  IPAddress gateway(192, 168, subNetIP, 1);
  IPAddress subnet(255, 255, 255, 0);
  IPAddress dns(192, 168, subNetIP, 1);
  WiFi.config(ip, dns, gateway, subnet); // auf feste IP einstellen
  WiFi.hostname(myHostname); //Hostnamen setzen
  WiFi.persistent(false);
  WiFi.mode(WIFI_OFF);
  WiFi.mode(WIFI_STA);
  connectToWiFi();
}

void reconnect() {
  // Loop bis zum Reconnect
  while (!client.connected()) {
    status = WiFi.status();
    if ( status != WL_CONNECTED) {
      connectToWiFi();
    }
    //Serial.print("Verbinde mit MQTT_Broker ...");
    if ( client.connect(clientID, TOKEN, NULL) ) {
      //Serial.println( "[Fertig]" );
    } else {
      //Serial.print( "[Fehler] [ rc = " );
      //Serial.print( client.state() );
      //Serial.println( " : neuer Versuch in 5 sek]" );
      // 5 Sekunden warten
      delay( 5000 );
    }
  }
}
